@@ -56,11 +56,20 @@ enum {
 G_DEFINE_TYPE (ChDeviceQueue, ch_device_queue, G_TYPE_OBJECT)
 
 typedef gboolean (*ChDeviceQueueParseFunc)	(guint8		*output_buffer,
+						 gsize		 output_buffer_size,
 						 gpointer	 user_data,
 						 GError		**error);
 
+typedef enum {
+	CH_DEVICE_QUEUE_DATA_STATE_PENDING,
+	CH_DEVICE_QUEUE_DATA_STATE_WAITING_FOR_HW,
+	CH_DEVICE_QUEUE_DATA_STATE_CANCELLED,
+	CH_DEVICE_QUEUE_DATA_STATE_COMPLETE,
+	CH_DEVICE_QUEUE_DATA_STATE_UNKNOWN
+} ChDeviceQueueDataState;
+
 typedef struct {
-	gboolean		 complete;
+	ChDeviceQueueDataState	 state;
 	GUsbDevice		*device;
 	guint8			 cmd;
 	guint8			*buffer_in;	/* we own this */
@@ -125,7 +134,7 @@ ch_device_queue_device_force_complete (ChDeviceQueue *device_queue, GUsbDevice *
 		data = g_ptr_array_index (device_queue->priv->data_array, i);
 		device_id_tmp = g_usb_device_get_platform_id (data->device);
 		if (g_strcmp0 (device_id_tmp, device_id) == 0)
-			data->complete = TRUE;
+			data->state = CH_DEVICE_QUEUE_DATA_STATE_CANCELLED;
 	}
 }
 
@@ -147,7 +156,8 @@ ch_device_queue_update_progress (ChDeviceQueue *device_queue)
 	/* find out how many commands are complete */
 	for (i = 0; i < device_queue->priv->data_array->len; i++) {
 		data = g_ptr_array_index (device_queue->priv->data_array, i);
-		if (data->complete)
+		if (data->state == CH_DEVICE_QUEUE_DATA_STATE_COMPLETE ||
+		    data->state == CH_DEVICE_QUEUE_DATA_STATE_CANCELLED)
 			complete++;
 	}
 
@@ -158,6 +168,26 @@ ch_device_queue_update_progress (ChDeviceQueue *device_queue)
 		       percentage);
 }
 
+/**
+ * ch_device_queue_count_in_state:
+ **/
+static guint
+ch_device_queue_count_in_state (ChDeviceQueue *device_queue,
+				ChDeviceQueueDataState state)
+{
+	guint i;
+	guint cnt = 0;
+	ChDeviceQueueData *data;
+
+	/* find any data objects in a specific state */
+	for (i = 0; i < device_queue->priv->data_array->len; i++) {
+		data = g_ptr_array_index (device_queue->priv->data_array, i);
+		if (data->state == state)
+			cnt++;
+	}
+	return cnt;
+}
+
 /**
  * ch_device_queue_process_write_command_cb:
  **/
@@ -173,6 +203,7 @@ ch_device_queue_process_write_command_cb (GObject *source,
 	gchar *error_msg = NULL;
 	GError *error = NULL;
 	guint i;
+	guint pending_commands;
 	GUsbDevice *device = G_USB_DEVICE (source);
 
 	/* mark it as not in use */
@@ -187,6 +218,7 @@ ch_device_queue_process_write_command_cb (GObject *source,
 	if (ret && data->parse_func != NULL) {
 		/* do any conversion function */
 		ret = data->parse_func (data->buffer_out,
+					data->buffer_out_len,
 					data->user_data,
 					&error);
 	}
@@ -214,6 +246,7 @@ ch_device_queue_process_write_command_cb (GObject *source,
 	}
 
 	/* update progress */
+	data->state = CH_DEVICE_QUEUE_DATA_STATE_COMPLETE;
 	ch_device_queue_update_progress (helper->device_queue);
 
 	/* is there another pending command for this device */
@@ -225,7 +258,12 @@ ch_device_queue_process_write_command_cb (GObject *source,
 	}
 out:
 	/* any more pending commands? */
-	if (g_hash_table_size (helper->device_queue->priv->devices_in_use) == 0) {
+	pending_commands = ch_device_queue_count_in_state (helper->device_queue,
+							   CH_DEVICE_QUEUE_DATA_STATE_PENDING);
+	pending_commands += ch_device_queue_count_in_state (helper->device_queue,
+							    CH_DEVICE_QUEUE_DATA_STATE_WAITING_FOR_HW);
+	g_debug ("Pending commands: %i", pending_commands);
+	if (pending_commands == 0) {
 
 		/* should we return the process with an error, or just
 		 * rely on the signal? */
@@ -241,6 +279,7 @@ out:
 		} else {
 			g_simple_async_result_set_op_res_gboolean (helper->res, TRUE);
 		}
+
 		/* remove all commands from the queue, as they are done */
 		g_ptr_array_set_size (helper->device_queue->priv->data_array, 0);
 		g_simple_async_result_complete_in_idle (helper->res);
@@ -263,7 +302,7 @@ ch_device_queue_process_data (ChDeviceQueueHelper *helper,
 	gboolean ret = FALSE;
 
 	/* is this command already complete? */
-	if (data->complete)
+	if (data->state == CH_DEVICE_QUEUE_DATA_STATE_COMPLETE)
 		goto out;
 
 	/* is this device already busy? */
@@ -292,7 +331,7 @@ ch_device_queue_process_data (ChDeviceQueueHelper *helper,
 	ret = TRUE;
 
 	/* remove this from the command queue -- TODO: retries? */
-	data->complete = TRUE;
+	data->state = CH_DEVICE_QUEUE_DATA_STATE_WAITING_FOR_HW;
 out:
 	return ret;
 }
@@ -451,6 +490,7 @@ ch_device_queue_add_internal (ChDeviceQueue		*device_queue,
 	g_return_if_fail (G_USB_IS_DEVICE (device));
 
 	data = g_new0 (ChDeviceQueueData, 1);
+	data->state = CH_DEVICE_QUEUE_DATA_STATE_PENDING;
 	data->parse_func = parse_func;
 	data->user_data = user_data;
 	data->cmd = cmd;
@@ -581,12 +621,24 @@ ch_device_queue_set_multiplier (ChDeviceQueue *device_queue,
  **/
 static gboolean
 ch_device_queue_buffer_uint16_from_le_cb (guint8 *output_buffer,
+					  gsize output_buffer_size,
 					  gpointer user_data,
 					  GError **error)
 {
+	gboolean ret = TRUE;
 	guint16 *tmp = (guint16 *) output_buffer;
+
+	/* check buffer size */
+	if (output_buffer_size != sizeof (guint16)) {
+		ret = FALSE;
+		g_set_error (error, 1, 0,
+			     "Wrong output buffer size, expected %" G_GSIZE_FORMAT ", got %" G_GSIZE_FORMAT,
+			     sizeof (guint16), output_buffer_size);
+		goto out;
+	}
 	*tmp = GUINT16_FROM_LE (*tmp);
-	return TRUE;
+out:
+	return ret;
 }
 
 /**
@@ -594,12 +646,24 @@ ch_device_queue_buffer_uint16_from_le_cb (guint8 *output_buffer,
  **/
 static gboolean
 ch_device_queue_buffer_uint32_from_le_cb (guint8 *output_buffer,
+					  gsize output_buffer_size,
 					  gpointer user_data,
 					  GError **error)
 {
+	gboolean ret = TRUE;
 	guint32 *tmp = (guint32 *) output_buffer;
+
+	/* check buffer size */
+	if (output_buffer_size != sizeof (guint32)) {
+		ret = FALSE;
+		g_set_error (error, 1, 0,
+			     "Wrong output buffer size, expected %" G_GSIZE_FORMAT ", got %" G_GSIZE_FORMAT,
+			     sizeof (guint32), output_buffer_size);
+		goto out;
+	}
 	*tmp = GUINT32_FROM_LE (*tmp);
-	return TRUE;
+out:
+	return ret;
 }
 
 /**
@@ -704,12 +768,23 @@ typedef struct {
  **/
 static gboolean
 ch_device_queue_buffer_to_firmware_ver_cb (guint8 *output_buffer,
+					   gsize output_buffer_size,
 					   gpointer user_data,
 					   GError **error)
 {
 	ChDeviceQueueGetFirmwareVerHelper *helper = (void *) user_data;
+	gboolean ret = TRUE;
 	guint16 *tmp = (guint16 *) output_buffer;
 
+	/* check buffer size */
+	if (output_buffer_size != sizeof (guint16) * 3) {
+		ret = FALSE;
+		g_set_error (error, 1, 0,
+			     "Wrong output buffer size, expected %" G_GSIZE_FORMAT ", got %" G_GSIZE_FORMAT,
+			     sizeof (guint16) * 3, output_buffer_size);
+		goto out;
+	}
+
 	*helper->major = GUINT16_FROM_LE (tmp[0]);
 	*helper->minor = GUINT16_FROM_LE (tmp[1]);
 	*helper->micro = GUINT16_FROM_LE (tmp[2]);
@@ -717,7 +792,8 @@ ch_device_queue_buffer_to_firmware_ver_cb (guint8 *output_buffer,
 	/* yes, we own this */
 	g_free (output_buffer);
 	g_free (helper);
-	return TRUE;
+out:
+	return ret;
 }
 
 /**
@@ -769,13 +845,24 @@ typedef struct {
  **/
 static gboolean
 ch_device_queue_buffer_to_get_calibration_cb (guint8 *output_buffer,
+					      gsize output_buffer_size,
 					      gpointer user_data,
 					      GError **error)
 {
 	ChDeviceQueueGetCalibrationHelper *helper = (void *) user_data;
+	gboolean ret = TRUE;
 	gdouble *calibration_tmp;
 	guint i;
 
+	/* check buffer size */
+	if (output_buffer_size != 60) {
+		ret = FALSE;
+		g_set_error (error, 1, 0,
+			     "Wrong output buffer size, expected %i, got %" G_GSIZE_FORMAT,
+			     60, output_buffer_size);
+		goto out;
+	}
+
 	/* convert back into floating point */
 	if (helper->calibration != NULL) {
 		calibration_tmp = cd_mat33_get_data (helper->calibration);
@@ -799,7 +886,8 @@ ch_device_queue_buffer_to_get_calibration_cb (guint8 *output_buffer,
 	/* yes, we own this */
 	g_free (output_buffer);
 	g_free (helper);
-	return TRUE;
+out:
+	return ret;
 }
 
 /**
@@ -1107,16 +1195,28 @@ ch_device_queue_clear_calibration (ChDeviceQueue *device_queue,
  **/
 static gboolean
 ch_device_queue_buffer_to_double_cb (guint8 *output_buffer,
+				     gsize output_buffer_size,
 				     gpointer user_data,
 				     GError **error)
 {
 	ChPackedFloat *buffer = (ChPackedFloat *) output_buffer;
+	gboolean ret = TRUE;
 	gdouble *value = (gdouble *) user_data;
 
+	/* check buffer size */
+	if (output_buffer_size != sizeof (ChPackedFloat)) {
+		ret = FALSE;
+		g_set_error (error, 1, 0,
+			     "Wrong output buffer size, expected %" G_GSIZE_FORMAT ", got %" G_GSIZE_FORMAT,
+			     sizeof (ChPackedFloat), output_buffer_size);
+		goto out;
+	}
+
 	/* convert back into floating point */
 	ch_packed_float_to_double (buffer, value);
 	g_free (output_buffer);
-	return TRUE;
+out:
+	return ret;
 }
 
 /**
@@ -1392,12 +1492,23 @@ ch_device_queue_write_eeprom (ChDeviceQueue *device_queue,
  **/
 static gboolean
 ch_device_queue_buffer_dark_offsets_cb (guint8 *output_buffer,
+					gsize output_buffer_size,
 					gpointer user_data,
 					GError **error)
 {
 	CdColorRGB *value = (CdColorRGB *) user_data;
+	gboolean ret = TRUE;
 	guint16 *buffer = (guint16 *) output_buffer;
 
+	/* check buffer size */
+	if (output_buffer_size != sizeof (guint16) * 3) {
+		ret = FALSE;
+		g_set_error (error, 1, 0,
+			     "Wrong output buffer size, expected %" G_GSIZE_FORMAT ", got %" G_GSIZE_FORMAT,
+			     sizeof (guint16) * 3, output_buffer_size);
+		goto out;
+	}
+
 	/* convert back into floating point */
 	value->R = (gdouble) buffer[0] / (gdouble) 0xffff;
 	value->G = (gdouble) buffer[1] / (gdouble) 0xffff;
@@ -1405,7 +1516,8 @@ ch_device_queue_buffer_dark_offsets_cb (guint8 *output_buffer,
 
 	/* yes, we own this */
 	g_free (buffer);
-	return TRUE;
+out:
+	return ret;
 }
 
 /**
@@ -1487,11 +1599,22 @@ ch_device_queue_take_reading_raw (ChDeviceQueue *device_queue,
  **/
 static gboolean
 ch_device_queue_buffer_triple_rgb_cb (guint8 *output_buffer,
+				      gsize output_buffer_size,
 				      gpointer user_data,
 				      GError **error)
 {
 	CdColorRGB *value = (CdColorRGB *) user_data;
 	ChPackedFloat *buffer = (ChPackedFloat *) output_buffer;
+	gboolean ret = TRUE;
+
+	/* check buffer size */
+	if (output_buffer_size != sizeof (ChPackedFloat) * 3) {
+		ret = FALSE;
+		g_set_error (error, 1, 0,
+			     "Wrong output buffer size, expected %" G_GSIZE_FORMAT ", got %" G_GSIZE_FORMAT,
+			     sizeof (ChPackedFloat) * 3, output_buffer_size);
+		goto out;
+	}
 
 	/* convert back into floating point */
 	ch_packed_float_to_double (&buffer[0], &value->R);
@@ -1500,7 +1623,8 @@ ch_device_queue_buffer_triple_rgb_cb (guint8 *output_buffer,
 
 	/* yes, we own this */
 	g_free (buffer);
-	return TRUE;
+out:
+	return ret;
 }
 
 /**
@@ -1534,11 +1658,22 @@ ch_device_queue_take_readings (ChDeviceQueue *device_queue,
  **/
 static gboolean
 ch_device_queue_buffer_triple_xyz_cb (guint8 *output_buffer,
+				      gsize output_buffer_size,
 				      gpointer user_data,
 				      GError **error)
 {
 	CdColorXYZ *value = (CdColorXYZ *) user_data;
 	ChPackedFloat *buffer = (ChPackedFloat *) output_buffer;
+	gboolean ret = TRUE;
+
+	/* check buffer size */
+	if (output_buffer_size != sizeof (ChPackedFloat) * 3) {
+		ret = FALSE;
+		g_set_error (error, 1, 0,
+			     "Wrong output buffer size, expected %" G_GSIZE_FORMAT ", got %" G_GSIZE_FORMAT,
+			     sizeof (ChPackedFloat) * 3, output_buffer_size);
+		goto out;
+	}
 
 	/* convert back into floating point */
 	ch_packed_float_to_double (&buffer[0], &value->X);
@@ -1547,7 +1682,8 @@ ch_device_queue_buffer_triple_xyz_cb (guint8 *output_buffer,
 
 	/* yes, we own this */
 	g_free (buffer);
-	return TRUE;
+out:
+	return ret;
 }
 
 /**
@@ -1651,6 +1787,7 @@ typedef struct {
  **/
 static gboolean
 ch_device_queue_buffer_read_flash_cb (guint8 *output_buffer,
+				      gsize output_buffer_size,
 				      gpointer user_data,
 				      GError **error)
 {
@@ -1658,6 +1795,15 @@ ch_device_queue_buffer_read_flash_cb (guint8 *output_buffer,
 	gboolean ret = TRUE;
 	guint8 expected_checksum;
 
+	/* check buffer size */
+	if (output_buffer_size != helper->len + 1) {
+		ret = FALSE;
+		g_set_error (error, 1, 0,
+			     "Wrong output buffer size, expected %" G_GSIZE_FORMAT ", got %" G_GSIZE_FORMAT,
+			     helper->len + 1, output_buffer_size);
+		goto out;
+	}
+
 	/* verify checksum */
 	expected_checksum = ch_device_queue_calculate_checksum (output_buffer + 1,
 								helper->len);
@@ -1721,6 +1867,7 @@ ch_device_queue_read_flash (ChDeviceQueue *device_queue,
  **/
 static gboolean
 ch_device_queue_buffer_verify_flash_cb (guint8 *output_buffer,
+					gsize output_buffer_size,
 					gpointer user_data,
 					GError **error)
 {
@@ -1728,6 +1875,15 @@ ch_device_queue_buffer_verify_flash_cb (guint8 *output_buffer,
 	gboolean ret = TRUE;
 	guint8 expected_checksum;
 
+	/* check buffer size */
+	if (output_buffer_size != helper->len + 1) {
+		ret = FALSE;
+		g_set_error (error, 1, 0,
+			     "Wrong output buffer size, expected %" G_GSIZE_FORMAT ", got %" G_GSIZE_FORMAT,
+			     helper->len + 1, output_buffer_size);
+		goto out;
+	}
+
 	/* verify checksum */
 	expected_checksum = ch_device_queue_calculate_checksum (output_buffer + 1,
 								helper->len);
@@ -1751,6 +1907,7 @@ ch_device_queue_buffer_verify_flash_cb (guint8 *output_buffer,
 	}
 out:
 	g_free (output_buffer);
+	g_free (helper->data);
 	g_free (helper);
 	return ret;
 }
@@ -1778,7 +1935,7 @@ ch_device_queue_verify_flash (ChDeviceQueue *device_queue,
 	/* create a helper structure as the checksum needs an extra
 	 * byte for the checksum */
 	helper = g_new0 (ChDeviceQueueReadFlashHelper, 1);
-	helper->data = (guint8 *) data;
+	helper->data = g_memdup (data, len + 1);
 	helper->len = len;
 	helper->address = address;
 
