@@ -29,6 +29,7 @@
 #include <gusb.h>
 #include <libsoup/soup.h>
 #include <colorhug.h>
+#include <canberra-gtk.h>
 
 #include "ch-markdown.h"
 #include "ch-flash-md.h"
@@ -65,6 +66,12 @@ ch_flash_error_dialog (ChFlashPrivate *priv, const gchar *title, const gchar *me
 	GtkWidget *widget;
 	GtkWidget *dialog;
 
+	/* play sound */
+	ca_context_play (ca_gtk_context_get (), 0,
+			 CA_PROP_EVENT_ID, "dialog-warning",
+			 CA_PROP_APPLICATION_NAME, _("ColorHug Updater"),
+			 CA_PROP_EVENT_DESCRIPTION, _("Calibration Failed"), NULL);
+
 	window = GTK_WINDOW(gtk_builder_get_object (priv->builder, "dialog_flash"));
 	dialog = gtk_message_dialog_new (window,
 					 GTK_DIALOG_MODAL,
@@ -217,6 +224,12 @@ ch_flash_set_flash_success_1_cb (GObject *source,
 	gtk_widget_set_sensitive (widget, TRUE);
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "button_flash"));
 	gtk_widget_hide (widget);
+
+	/* play sound */
+	ca_context_play (ca_gtk_context_get (), 0,
+			 CA_PROP_EVENT_ID, "complete",
+			 CA_PROP_APPLICATION_NAME, _("ColorHug Updater"),
+			 CA_PROP_EVENT_DESCRIPTION, _("Calibration Completed"), NULL);
 out:
 	return;
 }
@@ -1525,6 +1538,10 @@ ch_flash_startup_cb (GApplication *application, ChFlashPrivate *priv)
 		goto out;
 	}
 
+	/* automatically use the correct proxies */
+	soup_session_add_feature_by_type (priv->session,
+					  SOUP_TYPE_PROXY_RESOLVER_DEFAULT);
+
 	/* emulate a device */
 	if (g_getenv ("COLORHUG_EMULATE") != NULL) {
 		priv->device = ch_flash_get_fake_device (priv);
